##
## test_applicant.py
## Login : <uli@pu.smp.net>
## Started on  Tue Aug 24 04:36:11 2010 Uli Fouquet
## $Id$
## 
## Copyright (C) 2010 Uli Fouquet
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Tests for applicants and related.
"""
import grok
import unittest
from zope.component import adapts, queryUtility
from zope.component.interfaces import IFactory
from zope.interface import verify, implements
from zope.location.interfaces import ILocation
from waeup.sirp.image.interfaces import IWAeUPImageFile
from waeup.sirp.imagestorage import DefaultStorage
from waeup.sirp.interfaces import IFileStoreHandler, IFileStoreNameChooser
from waeup.sirp.applicants import (
    ResultEntry, Applicant, ApplicantFactory, get_regno_or_ac,
    ApplicantImageStoreHandler, ApplicantImageNameChooser,
    )
from waeup.sirp.applicants.interfaces import IResultEntry, IApplicant
from waeup.sirp.testing import FunctionalTestCase, FunctionalLayer

class FakeImageLocation(object):
    implements(ILocation)
    adapts(IWAeUPImageFile)
    def __init__(self, context):
        pass

class HelperTests(FunctionalTestCase):

    layer = FunctionalLayer

    def test_get_regno_or_ac(self):
        # we can get reg_no or access_code of an applicants if it is set
        appl1 = Applicant()
        appl2 = Applicant()
        appl2.reg_no = u'foo'
        appl3 = Applicant()
        appl3.access_code = u'bar'
        appl4 = Applicant()
        appl4.reg_no = u'foo'
        appl4.access_code = u'bar'
        self.assertTrue(
            get_regno_or_ac(appl1) is None)
        self.assertEqual(
            get_regno_or_ac(appl2), u'foo')
        self.assertEqual(
            get_regno_or_ac(appl3), u'bar')
        self.assertEqual(
            get_regno_or_ac(appl4), u'foo')
        return

    def test_image_store_handler_util_accessible(self):
        # we can get an IFileStoreHandler utility for applicants
        handler = queryUtility(IFileStoreHandler, name='img-applicant')
        self.assertTrue(
            isinstance(handler, ApplicantImageStoreHandler))
        return

    def test_image_store_handler(self):
        store = DefaultStorage()
        handler = queryUtility(IFileStoreHandler, name='img-applicant')
        result1 = handler.pathFromFileID(
            store, '/fake-root', '__img-applicant__sample.jpg')
        result2 = handler.pathFromFileID(
            store, '/fake-root', '__img-applicant__dir1/sample.jpg')
        result3 = handler.pathFromFileID(
            store, '/fake-root', '__img-applicant__dir1/dir2/sample.jpg')
        self.assertEqual(
            result1, '/fake-root/applicants/sample.jpg')
        self.assertEqual(
            result2, '/fake-root/applicants/dir1/sample.jpg')
        self.assertEqual(
            result3, '/fake-root/applicants/dir1/dir2/sample.jpg')
        return

class ApplicantImageNameChooserTests(FunctionalTestCase):

    layer = FunctionalLayer

    def test_iface(self):
        # make sure we implement promised interfaces
        obj = ApplicantImageNameChooser(None) # needs a context normally
        verify.verifyClass(IFileStoreNameChooser, ApplicantImageNameChooser)
        verify.verifyObject(IFileStoreNameChooser, obj)
        return

    def test_name_chooser_available(self):
        # we can get a name chooser for applicant objects as adapter
        appl = Applicant()
        chooser = IFileStoreNameChooser(appl)
        self.assertTrue(chooser is not None)
        return

    def test_name_chooser_applicant_wo_container(self):
        # we can get an image filename for applicants not in a container
        appl = Applicant()
        appl.reg_no = u'MY_REG_NO'
        chooser = IFileStoreNameChooser(appl)
        result = chooser.chooseName()
        # the file would be stored in a ``_default`` directory.
        self.assertEqual(
            result, '__img-applicant___default/MY_REG_NO.jpg')
        return

    def test_name_chooser_applicant_w_container(self):
        appl = Applicant()
        appl.reg_no = u'MY_REG_NO'
        fake_container = grok.Container()
        fake_container.__name__ = 'folder'
        fake_container['appl'] = appl
        appl.__parent__ = fake_container
        chooser = IFileStoreNameChooser(appl)
        result = chooser.chooseName()
        self.assertEqual(
            result, '__img-applicant__folder/MY_REG_NO.jpg')
        return

    def test_name_chooser_check_name(self):
        # we can check file ids for applicants
        appl = Applicant()
        appl.reg_no = u'MY_REG_NO'
        fake_container = grok.Container()
        fake_container.__name__ = 'folder'
        fake_container['appl'] = appl
        appl.__parent__ = fake_container
        chooser = IFileStoreNameChooser(appl)
        result1 = chooser.checkName('foo')
        result2 = chooser.checkName('__img-applicant__folder/MY_REG_NO.jpg')
        self.assertEqual(result1, False)
        self.assertEqual(result2, True)
        return

class ResultEntryTest(unittest.TestCase):

    def setUp(self):
        self.result_entry = ResultEntry()
        return

    def tearDown(self):
        pass

    def test_interfaces(self):
        verify.verifyClass(IResultEntry, ResultEntry)
        verify.verifyObject(IResultEntry, self.result_entry)

    def test_resultentry(self):
        entry = ResultEntry('Some subject', 3.7)
        assert entry.subject == 'Some subject'
        assert entry.score == 3.7

class ApplicantTest(FunctionalTestCase):

    layer = FunctionalLayer

    def setUp(self):
        super(ApplicantTest, self).setUp()
        self.applicant = Applicant()
        return

    def tearDown(self):
        super(ApplicantTest, self).tearDown()
        return

    def test_interfaces(self):
        verify.verifyClass(IApplicant, Applicant)
        verify.verifyObject(IApplicant, self.applicant)
        return

class ApplicantFactoryTest(FunctionalTestCase):

    layer = FunctionalLayer

    def setUp(self):
        super(ApplicantFactoryTest, self).setUp()
        self.factory = ApplicantFactory()
        return

    def tearDown(self):
        super(ApplicantFactoryTest, self).tearDown()
        return

    def test_interfaces(self):
        verify.verifyClass(IFactory, ApplicantFactory)
        verify.verifyObject(IFactory, self.factory)

    def test_factory(self):
        obj = self.factory()
        assert isinstance(obj, Applicant)

    def test_getInterfaces(self):
        implemented_by = self.factory.getInterfaces()
        assert implemented_by.isOrExtends(IApplicant)
